import json
import logging
from urllib.parse import urlencode

logger = logging.getLogger(__name__)


class BaseAPI:
    """BaseAPI"""

    def __init__(self, client):
        self._http = client.http
        self.group_id = client.group.group_id
        self._group = client.group
        self._account = client.account

    def _get(self, endpoint: str, payload: dict = None):
        """api _get"""
        return self._http.get(endpoint, payload)

    def _post(self, endpoint: str, payload: dict = None):
        """api _post"""
        return self._http.post(endpoint, payload)

    def _get_trx(self, trx_id: str):
        return self._get(f"/trx/{self.group_id}/{trx_id}")

    def _get_content(self, payload: dict):
        for k, v in payload.items():
            if isinstance(v, bool):
                payload[k] = json.dumps(v)
        query_string = urlencode(payload, doseq=True)
        return self._get(f"/node/{self.group_id}/groupctn?{query_string}")

    def _post_content(self, payload: dict):
        return self._post(f"/node/{self.group_id}/trx", payload)

    def _get_group(self):
        return self._get(f"/node/{self.group_id}/info")

    def _get_encryptpubkeys(self):
        return self._get(f"/node/{self.group_id}/encryptpubkeys")

    def _get_auth_type(self, trx_type: str):
        return self._get(f"/node/{self.group_id}/auth/by/{trx_type}")

    def _get_alwlist(self):
        return self._get(f"/node/{self.group_id}/auth/alwlist")

    def _get_denylist(self):
        return self._get(f"/node/{self.group_id}/auth/denylist")

    def _get_appconfig_keylist(self):
        return self._get(f"/node/{self.group_id}/appconfig/keylist")

    def _get_appconfig_key(self, key: str):
        return self._get(f"/node/{self.group_id}/appconfig/by/{key}")

    def _get_producers(self):
        return self._get(f"/node/{self.group_id}/producers")

    def _get_announced_producer(self):
        return self._get(f"/node/{self.group_id}/announced/producer")

    def _get_announced_user(self):
        return self._get(f"/node/{self.group_id}/announced/user")

    def _post_announce(self, payload: dict):
        return self._post(f"/node/{self.group_id}/announce", payload)



import base64
import logging
from typing import Union

import eth_keys
from eth_account import Account
from eth_utils.hexadecimal import encode_hex

logger = logging.getLogger(__name__)


def create_pvtkey() -> str:
    """create private key"""
    acc = Account().create()
    private_key = encode_hex(acc.key)
    return private_key


def check_pvtkey(private_key: Union[str, int, bytes]) -> bytes:
    """check private key"""
    if isinstance(private_key, int):
        private_key = hex(private_key)
    if isinstance(private_key, str):
        if private_key.startswith("0x"):
            private_key = bytes.fromhex(private_key[2:])
        else:
            private_key = bytes.fromhex(private_key)
    elif isinstance(private_key, bytes):
        pass
    else:
        raise ValueError("Invalid private key. param private_key is required.")
    if len(private_key) != 32:
        raise ValueError("Invalid private key. param private_key is required.")
    return private_key


def pvtkey_to_address(private_key: Union[str, int, bytes]) -> str:
    """private key to address"""
    private_key = check_pvtkey(private_key)
    address = Account().from_key(private_key).address
    return address


def pubkey_to_address(public_key: str) -> str:
    """public key to address"""
    public_key = base64.urlsafe_b64decode(public_key)
    pubkey = eth_keys.keys.PublicKey.from_compressed_bytes(public_key)
    address = pubkey.to_checksum_address()
    return address


def pvtkey_to_pubkey(private_key: Union[str, int, bytes]) -> str:
    """private key to public key"""
    private_key = check_pvtkey(private_key)
    account = eth_keys.keys.PrivateKey(private_key)
    public_key = base64.urlsafe_b64encode(
        account.public_key.to_compressed_bytes()
    ).decode()
    return public_key


def pvtkey_to_keystore(private_key: Union[str, int, bytes], password: str):
    """private key to keystore with password"""
    private_key = check_pvtkey(private_key)
    keystore = Account().from_key(private_key).encrypt(password=password)
    return keystore


def keystore_to_pvtkey(keystore: dict, password: str) -> str:
    """keystore with password to private key"""
    pvtkey = Account().decrypt(keystore, password)
    private_key = encode_hex(pvtkey)
    return private_key



import logging
import os

from pyrage import decrypt, encrypt, passphrase, x25519

logger = logging.getLogger(__name__)


def create_age_keypair():
    """create age private key"""
    age_identity = x25519.Identity.generate()
    age_pvtkey = str(age_identity)
    age_pubkey = str(age_identity.to_public())
    return age_pvtkey, age_pubkey


def age_pvtkey_to_pubkey(age_pvtkey: str) -> str:
    """age private key to public key"""
    age_identity = x25519.Identity.from_str(age_pvtkey)
    age_pubkey = str(age_identity.to_public())
    return age_pubkey


def age_encrypt(recipients: list, data: bytes) -> bytes:
    """recipients list of x25519.Recipient"""
    if not data:
        raise ValueError("invalid data")
    if not recipients:
        raise ValueError("invalid recipients")

    _recipients = []
    for item in recipients:
        if isinstance(item, str):
            _recipients.append(x25519.Recipient.from_str(item))
        else:
            _recipients.append(item)

    return encrypt(data, _recipients)


def age_decrypt(recipient: x25519.Recipient, data: bytes):
    if isinstance(recipient, str):
        recipient = x25519.Recipient.from_str(recipient)
    return decrypt(data, [recipient])


def age_pubkey_from_str(pubkey: str) -> x25519.Recipient:
    return x25519.Recipient.from_str(pubkey)


def age_privkey_from_str(key: str) -> x25519.Identity:
    identity = x25519.Identity.from_str(key)
    return identity


def age_privkey_from_file(path: str, password: str) -> x25519.Identity:
    if not os.path.exists(path):
        raise ValueError(f"can not find file path: {path}")

    with open(path, "rb") as fp:
        decrypted = passphrase.decrypt(fp.read(), password)
        return age_privkey_from_str(str(decrypted))



import logging

from quorum_mininode_py.client import MiniNode, RumAccount, RumGroup
from quorum_mininode_py.crypto.account import *
from quorum_mininode_py.utils.url import decode_seed_url, update_seed_url

__version__ = "1.2.8"
__author__ = "liujuanjuan1984, zhangwm404"


logger = logging.getLogger(__name__)
logger.info("Version: %s", __version__)



import logging

from quorum_mininode_py.api.base import BaseAPI
from quorum_mininode_py.crypto.account import check_pvtkey
from quorum_mininode_py.crypto.trx import trx_decrypt, trx_encrypt

logger = logging.getLogger(__name__)


class LightNodeAPI(BaseAPI):
    """the light node api for quorum"""

    def post_content(
        self,
        data: dict,
        trx_id: str = None,
    ):
        """post content to group"""
        if self._group.encryption_type == "public":
            age_pubkey = None
        else:
            age_pubkey = self._account.age_pubkey
        trx = trx_encrypt(
            self.group_id,
            self._group.aes_key,
            data,
            check_pvtkey(self._account.pvtkey),
            age_pubkey,
            trx_id,
        )
        return super()._post_content(trx)

    def get_trx(self, trx_id: str):
        """get encrpyted trx"""
        return super()._get_trx(trx_id)

    def trx(self, trx_id: str):
        """get decrypted trx"""
        if self._group.encryption_type == "public":
            age_priv_key = None
        else:
            age_priv_key = self._account.age_privkey

        trx = trx_decrypt(self._group.aes_key, age_priv_key, self.get_trx(trx_id))
        return trx

    def get_content(
        self,
        start_trx: str = None,
        num: int = 20,
        reverse: bool = False,
        include_start_trx: bool = False,
        senders: list = None,
    ):
        """get content"""
        params = {
            "reverse": reverse,
            "num": num,
        }
        if start_trx:
            params["start_trx"] = start_trx
            params["include_start_trx"] = include_start_trx
        if senders:
            params["senders"] = senders

        encypted_trxs = super()._get_content(params)
        if self._group.encryption_type.lower() == "public":
            age_priv_key = None
        else:
            age_priv_key = self._account.age_privkey

        trxs = []
        for i in encypted_trxs:
            try:
                i = trx_decrypt(self._group.aes_key, age_priv_key, i)
            except Exception as err:
                logger.error(err)
            trxs.append(i)
        return trxs

    def get_group_info(self):
        """get group info"""
        return super()._get_group()

    def get_encryptpubkeys(self):
        """get encrypt pubkeys"""
        return super()._get_encryptpubkeys()

    def get_auth_type(self, trx_type: str):
        """get auth type of trx_type"""
        trx_type = trx_type.upper()
        if trx_type not in ["POST", "ANNOUNCE", "REQ_BLOCK"]:
            raise ValueError("trx_type must be one of [POST ANNOUNCE REQ_BLOCK]")
        return super()._get_auth_type(trx_type)

    def get_auth_allowlist(self):
        """get allowlist"""
        return super()._get_alwlist()

    def get_auth_denylist(self):
        """get denylist"""
        return super()._get_denylist()

    def get_appconfig_keylist(self):
        """get appconfig keylist"""
        return super()._get_appconfig_keylist()

    def get_appconfig_key(self, key: str):
        """get appconfig key value by keyname"""
        return super()._get_appconfig_key(key)

    def get_producers(self):
        """get group producers"""
        return super()._get_producers()

    def get_announced_producer(self):
        """get announced producer"""
        return super()._get_announced_producer()

    def get_announced_user(self):
        """get announced user"""
        return super()._get_announced_user()

    def announce(self, payload: dict):  # TODO: to finish.
        """post annonce to group"""
        return super()._post_announce(payload)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="quorum_mininode_py",
    version="1.2.8",
    author="liujuanjuan1984, zhangwm404",
    author_email="qiaoanlu@163.com",
    description="a mini python sdk for quorum lightnode with http/https requests to quorum fullnode",
    keywords=["rumsystem", "quorum", "lightnode", "sdk"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/liujuanjuan1984/quorum_mininode_py",
    project_urls={
        "Github Repo": "https://github.com/liujuanjuan1984/quorum_mininode_py",
        "Bug Tracker": "https://github.com/liujuanjuan1984/quorum_mininode_py/issues",
        "About Quorum": "https://github.com/rumsystem/quorum",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(exclude=["example"]),
    python_requires=">=3.5",
    install_requires=[
        "requests",
        "eth_keys",
        "pyrage",
        "protobuf",
        "eth_account>=0.8.0",
    ],
)



import base64
import hashlib
import json
import logging
import time
import uuid
from typing import Any, Dict, Union

import eth_keys
from pyrage import x25519

from quorum_mininode_py.crypto.aes import aes_decrypt, aes_encrypt
from quorum_mininode_py.crypto.age import age_decrypt, age_encrypt
from quorum_mininode_py.proto import pbQuorum

logger = logging.getLogger(__name__)


def age_privkey_from_str(key: str) -> x25519.Identity:
    identity = x25519.Identity.from_str(key)
    return identity


def pack_obj(obj: Dict[str, str], aes_key) -> str:
    """pack obj with group cipher_key and return a string"""
    obj_bytes = json.dumps(obj).encode()
    obj_encrypted = aes_encrypt(aes_key, obj_bytes)
    req = base64.b64encode(obj_encrypted).decode()
    return req


def trx_encrypt(
    group_id: str,
    aes_key: bytes,
    data: Dict[str, Any] = None,
    private_key: bytes = None,
    age_pubkey=None,
    trx_id=None,
) -> Dict[str, str]:
    """trx encrypt"""
    # pylint: disable=too-many-locals

    data = json.dumps(data).encode()
    encrypted = None
    if not age_pubkey:
        encrypted = aes_encrypt(aes_key, data)
    else:
        encrypted = age_encrypt(age_pubkey, data)

    account = eth_keys.keys.PrivateKey(private_key)
    sender_pubkey = base64.urlsafe_b64encode(
        account.public_key.to_compressed_bytes()
    ).decode()

    trx = {
        "TrxId": trx_id or str(uuid.uuid4()),
        "GroupId": group_id,
        "Data": encrypted,
        "TimeStamp": int(time.time() * 1e9),
        "Version": "2.0.0",
        "SenderPubkey": sender_pubkey,
    }

    trx_without_sign_pb = pbQuorum.Trx(**trx)  # pylint: disable=no-member
    trx_without_sign_pb_bytes = trx_without_sign_pb.SerializeToString()
    trx_hash = hashlib.sha256(trx_without_sign_pb_bytes).digest()
    signature = account.sign_msg_hash(trx_hash).to_bytes()
    trx["SenderSign"] = signature

    for k, v in trx.items():
        if isinstance(v, bytes):
            trx[k] = base64.b64encode(v).decode()

    trx = {
        "trx_id": trx["TrxId"],
        "data": trx["Data"],
        "timestamp": str(trx["TimeStamp"]),
        "version": trx["Version"],
        "sender_pubkey": trx["SenderPubkey"],
        "sender_sign": trx["SenderSign"],
    }
    return trx


def _check_data(data):
    trx_data = b""
    if isinstance(data, str):
        trx_data = data.encode()
    elif isinstance(data, bytes):
        trx_data = data
    return trx_data


def decode_public_trx_data(aes_key: bytes, data: str):
    trx_data = _check_data(data)
    trx_enc_bytes = base64.b64decode(trx_data)
    trx_bytes = aes_decrypt(aes_key, trx_enc_bytes)
    return trx_bytes


def decode_private_trx_data(age_key: str, data: str):
    trx_data = _check_data(data)
    trx_enc_bytes = base64.b64decode(trx_data)
    age_key = age_privkey_from_str(age_key)
    trx_bytes = age_decrypt(age_key, trx_enc_bytes)
    return trx_bytes


def trx_decrypt(
    aes_key: Union[bytes, None], age_priv_key: Union[str, None], encrypted_trx: dict
):

    data = encrypted_trx.get("Data")
    if data is None:
        raise ValueError("Data is None")
    trx_enc_bytes = base64.b64decode(data)
    trx_bytes = None
    if aes_key:
        trx_bytes = aes_decrypt(aes_key, trx_enc_bytes)
    elif age_priv_key:
        age_key = age_privkey_from_str(age_priv_key)
        trx_bytes = age_decrypt(age_key, trx_enc_bytes)
    else:
        raise ValueError("aes_key and age_key both empty")

    return {**encrypted_trx, "Data": json.loads(trx_bytes)}


def get_sender_pubkey(private_key: bytes) -> str:
    pk = eth_keys.keys.PrivateKey(private_key)
    return base64.urlsafe_b64encode(pk.public_key.to_compressed_bytes()).decode()
